# -*- coding: utf-8 -*-

from typing import List
from icemammoth_common.util.log_util import logger


def fetchObjectAttributes(obj) -> List[str]:
    return [
        attribute
        for attribute in dir(obj)
        if not attribute.startswith("__") and not callable(getattr(obj, attribute))
    ]
    
def existAttribute(obj, attrName):
    return attrName in fetchObjectAttributes(obj)

def getObjectAttributesAndValues(obj):
    attributes = [
        attribute
        for attribute in dir(obj)
        if not attribute.startswith("__") and not callable(getattr(obj, attribute))
    ]
    return {attribute: getattr(obj, attribute) for attribute in attributes}

def attrsLessThan(*valuePairs):
    for valuePair in valuePairs:
        if valuePair[0] == valuePair[1]:
            continue
        if valuePair[0] is None:
            return True
        return False if valuePair[1] is None else valuePair[0] < valuePair[1]
    return True


def shallowCopyAttribute(source, dest):
    sourceAttrValuesMap = getObjectAttributesAndValues(source)
    attributes = [
        attribute
        for attribute in dir(dest)
        if not attribute.startswith("__") and not callable(getattr(dest, attribute))
    ]
    for attribute in attributes:
        if sourceValue := sourceAttrValuesMap.get(attribute):
            setattr(dest,attribute,sourceValue)




# -*- coding: utf-8 -*-

import io
from PIL import Image

def convertToJPGImageBytes(imageBytes:bytes) -> bytes:
    # 打开图像
    image = Image.open(io.BytesIO(imageBytes))
    # 将图像转换为JPG格式
    jpgImageBytesIO = io.BytesIO()
    image.convert("RGB").save(jpgImageBytesIO,'JPEG')
    return jpgImageBytesIO.getvalue()

def fetchImageType(imageBytes:bytes) -> str:
        # typeMap = {
        #      'FFD8':'JPG',
        #      '424D':'BMP',
        #      '474946':'GIF',
        #      '89504E470D0A1A0A':'PNG',
        # }
        # sig = ""
        # for i in range(8):
        #     sig += hex(imageBytes[i]).upper()[2:]
        #     imageType = typeMap.get(sig)
        #     if imageType:
        #          return imageType
        
        # return None
        '''
            Image format

            RGB : SGI ImgLib Files
            GIF : GIF 87a and 89a Files
            PBM : Portable Bitmap Files
            PGM : Portable Graymap Files
            PPM : Portable Pixmap Files
            TIFF : TIFF Files
            RAST : Sun Raster Files
            XBM : X Bitmap Files
            JPEG : JPEG data in JFIF or Exif formats
            BMP : BMP files
            PNG : Portable Network Graphics
            WEBP : WebP files
            EXR : OpenEXR Files
        '''
        image = Image.open(io.BytesIO(imageBytes))
        return image.format
   
def fetchImageSize(imageBytes:bytes) -> (int,int):
    image =  Image.open(io.BytesIO(imageBytes))
    return (image.width,image.height)

def resetImageSize(imageBytes:bytes,width:int,height:int) -> bytes:
     image = Image.open(io.BytesIO(imageBytes))
     imageType = image.format
     jpgImageBytesIO = io.BytesIO()
     image.resize((width,height),Image.Resampling.NEAREST).save(jpgImageBytesIO, imageType , optimize=True, quality=95)
     return jpgImageBytesIO.getvalue()


# -*- coding: utf-8 -*-

import math
from icemammoth_common.util.log_util import logger

def str_to_float(s,default = None):
    if s is None:
        return default
    try:
        if type(s) == str:
            s = s.strip().replace(',','')
        return float(s)
    except ValueError:
        logger.warning(f'{s} is illegal float, return None')
        return None
    
def str_to_int(s):
    if s is None:
        return None
    
    try:
        return int(s.strip().replace(',',''))
    except ValueError:
        logger.warning(f'{s} is illegal int, return None')
        return None
    
def base26_to_decimal(s):
    s = s.upper()
    num = 0
    for i in s:
        num = num * 26 + ord(i) - ord('A')
    return num
    
def decimal_to_base26(num):
    s = ''
    while num > 0:
        s = chr(num % 26 + ord('A')) + s
        num //= 26
    return s

def convertByPrecision(f:float, precision:int) -> float:
    return round(f,precision) if precision >= 0 else math.trunc(f/math.pow(10,-precision)) * math.pow(10,-precision)

def roundToInt(number) -> int:
    if type(number) == float:
        return int(round(number,1))
    elif type(number) == int:
        return number
    elif type(number) == str:
        return int(round(float(number),1))
    else:
        raise TypeError(f'number is not valid type converting to int!number:{number},type:{type(number)}')
    
def calculate_pages(total_items, pagesize):
    pages = (total_items + pagesize - 1) // pagesize
    return pages


# -*- coding: utf-8 -*-

from mysql.connector import pooling
from icemammoth_common.util import module_util
from icemammoth_common.base.singleton import SingletonMeta
from icemammoth_common.util import module_util
from icemammoth_common.util import log_util

CONFIG_MODULE_NAME = "config"

class ConnectPool(metaclass=SingletonMeta):

    def __init__(self):

        if not module_util.module_exist(CONFIG_MODULE_NAME):
            log_util.logger.exception(
                "config module is not exist, must set config module first!")
            raise Exception(
                "config module is not exist, must set config module first!")

        datebase_host = module_util.get_variable(
            CONFIG_MODULE_NAME, "DATABASE_HOST", except_unexist=True)
        database_port = module_util.get_variable(
            CONFIG_MODULE_NAME, "DATABASE_PORT", except_unexist=True)
        database_user = module_util.get_variable(
            CONFIG_MODULE_NAME, "DATABASE_USER", except_unexist=True)
        database_password = module_util.get_variable(
            CONFIG_MODULE_NAME, "DATABASE_PASSWORD", except_unexist=True)
        database_db_name = module_util.get_variable(
            CONFIG_MODULE_NAME, "DATABASE_DB_NAME", except_unexist=True)
        database_connect_pool_size = module_util.get_variable(
            CONFIG_MODULE_NAME, "DATABASE_CONNECT_POOL_SIZE", except_unexist=True)

        # dbconfig = {
        #     "host": datebase_host,
        #     "port": database_port,
        #     "user": database_user,
        #     "password": database_password,
        #     "database":database_db_name,
        # }

        self.cnxpool = pooling.MySQLConnectionPool(host=datebase_host,
                                                   port=database_port,
                                                   user=database_user,
                                                   password=database_password,
                                                   database=database_db_name,
                                                   pool_name="connection-pool",
                                                   pool_reset_session=True,
                                                   pool_size=database_connect_pool_size,
                                                   autocommit=True)

    def getConnection(self):
        return self.cnxpool.get_connection()



# -*- coding: utf-8 -*-

import inspect,sys,os,time
from factory import FactoryProduction
from icemammoth_common.util.log_util import logger

def fetchAllModuleNames(module_name):
    module = sys.modules[module_name]
    module_dir = os.path.dirname(module.__file__)
    module_names = []

    for x in os.walk(module_dir):
        x_module_dir = x[0]
        if str.endswith(x_module_dir,'/__pycache__'):
            continue

        relative_dir = x_module_dir[len(module_dir):]
        dir_sub_module = module_name+'.'.join(relative_dir.split('/'))
        module_names.append(dir_sub_module)

        file_names = x[2]
        for file_name in file_names:
            if file_name == '__init__.py' :
                continue
            if not str.endswith(file_name,'.py'):
                continue

            module_names.append(f"{dir_sub_module}.{file_name[:-3]}")

    return module_names

def registerProductionsByModule(module_name):
    if module_name not in sys.modules:
        __import__(module_name, globals(), locals(), [module_name.split('.')[-1]])
    module = sys.modules[module_name]
    members = inspect.getmembers(module)
    classes = []

    productionClasses = set()
    for cls_name, cls_obj in members:
        
        if not inspect.isclass(cls_obj):
            continue
        
        classes.append(cls_name)
        productionClass = getattr(module, cls_name)
        
        if not issubclass(productionClass,FactoryProduction) or productionClass == FactoryProduction:
            continue
        
        productionClasses.add(productionClass)
    
    for productionClass in productionClasses:
        productionClass()

def registerProductionsInModule(module_name):
    start_time = time.time()
    module_names = fetchAllModuleNames(module_name)
    for module_name in module_names:
        # logger.info(f'register factory productions by module {module_name}')
        registerProductionsByModule(module_name)
    end_time = time.time()
    logger.info(f'register factory production cost time:{end_time-start_time}s')


# -*- coding: utf-8 -*-

import logging
from typing import Any, Dict
import coloredlogs
import sys

from icemammoth_common.util import module_util

CONFIG_MODULE_NAME = "config"

log_level: int = logging.INFO

logger: logging.Logger = logging.getLogger(__name__)

# color类型
# colors:{'black','blue','cyan','green','magenta','red','white','yellow'}

# 字符样式
# text_styles:{'bold','bright','faint','inverse','normal','strike_through','underline'}

# 定义日志字段样式的字典
field_style: Dict[str, Any] = {'asctime': {'color': 'green'},  # 时间戳字段样式，使用绿色文本
                               # 日志级别字段样式，使用加粗的黑色文本
                               'levelname': {'bold': True, 'color': 'black'},
                               'filename': {'color': 'cyan'},  # 文件名字段样式，使用青色文本
                               'funcName': {'color': 'blue'},  # 函数名字段样式，使用蓝色文本
                               }

# 定义不同日志级别样式的字典
level_styles: Dict[str, Any] = {'critical': {'bold': True, 'color': 'red'},  # 致命错误级别样式，使用加粗的红色文本
                                'debug': {'color': 'green'},  # 调试级别样式，使用绿色文本
                                'error': {'color': 'red'},  # 错误级别样式，使用红色文本
                                'info': {'color': 'cyan'},  # 信息级别样式，使用青色文本
                                # 警告级别样式，使用黄色文本
                                'warning': {'color': 'yellow'},
                                }
"""
此函数调用`coloredlogs.install()`来配置并激活一个带有颜色的日志记录器。
日志格式如下：

- 时间戳: %(asctime)s
- 级别: [%(levelname)s]
- 文件名: %(filename)s
- 函数名: %(funcName)s
- 行号: %(lineno)s
- 消息: %(message)s

日志时间戳格式化为 '%Y-%m-%d %H:%M:%S'。

此外，还可以通过`field_styles`和`level_styles`参数定制字段和级别样式。

@params:
- level: 日志级别，例如 logging.DEBUG, logging.INFO 等。
- logger: 要配置的日志器实例。
- fmt: 定制的日志消息格式字符串。
- datefmt: 时间戳的自定义日期时间格式。
- field_styles: 字段样式的字典，用于指定颜色和其他样式。
- level_styles: 级别样式的字典，用于指定不同日志级别颜色和其他样式。

@example:
coloredlogs.install(level=logging.INFO, logger=my_logger, ...)
"""
coloredlogs.install(level=log_level,
                    logger=logger,
                    fmt='%(asctime)s [%(levelname)s] [%(filename)s:%(funcName)s:%(lineno)s] - %(message)s',
                    datefmt='%Y-%m-%d %H:%M:%S',
                    field_styles=field_style,
                    level_styles=level_styles)


if CONFIG_MODULE_NAME not in sys.modules:
    # 如果模块没有加载，则 warning日志级别
    logger.warning(f"[Log Boot Info] not found config module! set default log config!")
elif not module_util.has_variable(CONFIG_MODULE_NAME, 'LOG_LEVEL') and not module_util.has_variable(CONFIG_MODULE_NAME, 'LOG_LEVEL'):
    # 如果没有，则 warning日志级别
    logger.warning("[Log Boot Info] LOG_LEVEL/log_level not set in {config_module} module! set default log config!")
else:
    # 获取LOG_LEVEL的值
    log_level = module_util.get_variable(CONFIG_MODULE_NAME, 'LOG_LEVEL')
    if not log_level:
        # 如果没有，则获取log_level的值
        log_level = module_util.get_variable(CONFIG_MODULE_NAME, 'log_level')

# 打印日志启动信息
logger.info(f"[Log Boot Info] set log level: {logging.getLevelName(log_level)}.")
# 设置日志级别
logger.setLevel(log_level)


# 重新调整设置日志级别
coloredlogs.adjust_level(logger, level=log_level)



import os,io
from pathlib import Path
from icemammoth_common.util.log_util import logger

class DefaultDirFilesProcessor(object):
    
    def processDir(self, dir,subDir):
        print(f'Processing directory dir:{dir},sub_dir:{subDir}')
    
    def processFile(self, dir,file):
        print(f'Processing line dir:{dir},file:{file}')

def loop_dir(dir, dirProcessFunc=None, fileProcessFunc=None, recursive=True):

    for root, dirs, files in os.walk(dir, topdown=True):
            
        if dirProcessFunc:
            for subDir in dirs:
                dirProcessFunc(root,subDir)
                if recursive:
                    loop_dir(os.path.join(root,subDir), dirProcessFunc, fileProcessFunc)
        if fileProcessFunc:
            for file in files:
                fileProcessFunc(root,file)

class CleanDirProcessor(object):
    
    def processDir(self, dir,subDir):
        os.rmdir(os.path.join(dir, subDir))
    
    def processFile(self, dir,file):
        os.remove(os.path.join(dir, file))

'''
    清理目录
    create_not_exist:目录不存在时是否创建
'''
def clean_dir(dir, create_not_exist=True):
    dir: str = rf"{dir}"
    if not os.path.exists(dir):
        if create_not_exist:
            logger.info(f"[INFO] not exist directory {dir}, create it.")
            os.makedirs(dir)
        return
    loop_dir(dir,CleanDirProcessor())

'''
    如果目录不存在创建目录
'''
def create_dir_if_not_exist(dir):
    dir: str = rf"{dir}"
    if not os.path.exists(dir):
        logger.info(f"[INFO] not exist directory {dir}, create it.")
        try:
            os.makedirs(dir)
        except FileExistsError:
            return
        except Exception as e:
                raise e


'''
    清理目录
    create_not_exist:目录不存在时是否创建
'''
def reset_dir(dir,create_not_exist=True):
    clean_dir(dir,create_not_exist)
    
'''
    删除文件
    create_not_exist:当文件目录不存在时是否创建
'''
def remove_file(file, create_dir_not_exist=True):
    # sourcery skip: avoid-builtin-shadow
    dir = os.path.dirname(file)
    if not os.path.exists(dir):
        if create_dir_not_exist:
            os.makedirs(dir,exist_ok=True)
    elif os.path.exists(file):
        os.remove(file)

'''
    将内容写入文件
'''
def dumpToFile(path,content):
    remove_file(path,True)
    with open(path,'w') as file:
        file.write(content)
        file.flush()

class DefaultFileReadProcessor(object):
    
    def processLine(self, filePath, index, line):
        logger.info(f'Processing file:{filePath}, index:{index},line:{line[:16]}')


class FileContentReadProcessor(object):


    def __init__(self) -> None:
        self.content = ""
    
    def processLine(self, filePath, index, line):
        logger.info('process line: %s' % line)
        self.content = self.content + line +'\n'

    def getContent(self):
        return self.content

'''
    读取文件,并对其内容进行处理
    processor:文件数据处理类
'''
def readFile(filepath, processor=DefaultFileReadProcessor()):
    # sourcery skip: raise-specific-error
    
    with io.open(filepath, 'r') as file:
        index = 0
        try:
            lines: list[str] = file.readlines()
            for line in lines:
                logger.debug('Read line: %s' % line)
                if line.endswith("\n"):
                    line = line[:-1]
                processor.processLine(filepath, index, line)
                index += 1
        except Exception as err:
            logger.exception(f'read and process file error! file:{filepath},processor:{type(processor)}')

            raise Exception(
                f'read and process file error! file:{filepath},processor:{type(processor)}'
            ) from err
        
def saveMapData(filePath, mapData):
    content = ''
    for k,v in mapData.items():
        line = f'{k}:{v}'
        content = content + line + '\n'
    dumpToFile(filePath, content)

# -*- coding: utf-8 -*-

class MapDataReader(object):

    def __init__(self):
        self.mapData = {}
    
    def processLine(self, filePath, index, line):
        firstColonIndex = line.find(':')
        if firstColonIndex > -1:
            sline = line.strip()
            key = sline[:firstColonIndex]
            value = sline[firstColonIndex+1:]
            self.mapData[key] = value
        
    def getMapData(self):
        return self.mapData
    
def loadMapData(filepath, exception_on_file_noexist=True):
    if not exception_on_file_noexist and not os.path.exists(filepath):
        return {}
        
    mapDataReader = MapDataReader()
    readFile(filepath, processor=mapDataReader)
    return mapDataReader.getMapData()

'''
    将内容写入文件
'''
def dumpBytesToFile(path,contentBytes):
    remove_file(path,True)
    with open(path,'wb') as file:
        # file.write(BytesIO(contentBytes).getbuffer())
        # file.flush()
        file.write(contentBytes)

def appendToFile(path,content) -> None:
    with open(path,'a') as file:
        file.write(content)

'''
    读取文件内容,并对其内容进行处理
    processor:文件数据处理类
'''
def readFileCotent(filepath :str, processor=DefaultFileReadProcessor()) -> str:
    
    processor = FileContentReadProcessor()
    readFile(filepath, processor)
    return processor.getContent()

'''
    获取文件扩展名
'''
def fetchFileExtension(file):
    fullpath = Path(file)
    return fullpath.suffix

'''
    获取文件名不包含扩展名
'''
def fetchFileNameWithoutExtension(file):
    fullpath = Path(file)
    return fullpath.stem

'''
    获取文件名包含扩展名
'''
def fetchFileNameWithExtension(file):
    fullpath = Path(file)
    return fullpath.name


# -*- coding: utf-8 -*-

from icemammoth_common.base.singleton import SingletonMeta
from icemammoth_common.util.log_util import logger

class FactoryProduction(type):

    def __new__(cls,*args,**kwargs):
        '''
        产生一个新类
        '''
        instance=super().__new__(cls,cls.__name__,args,kwargs)
        cls.__register_model_entity(instance)
        return instance
 
    def __init__(self):
        pass
 
    @classmethod
    def __register_model_entity(cls, instance):
        instanceType = instance.getType()
        if instanceType is None:
            return
        factory = cls.getFactory()
        factory.register(type,instance)

    @classmethod
    def getFactory():
        pass

    def getType(self):
        pass


class Factory(metaclass=SingletonMeta):
    
    '''
    Factory is a singleton class, which is used to register and create production

    工厂模版是一个单例类，用于注册和创建产品, 通过继承该类的方式获得工厂模式能力
    '''

    def __init__(self):
        # 产品类型和产品之间的映射关系
        self.productions: dict = {}

    def register(self, type : any, production: FactoryProduction) -> None:
        '''
        注册产品类型和产品

        参数:
            type: 产品类型
            production: 产品
        '''
        if type not in self.productions:
            logger.info(f'{self.__class__.__module__}.{self.__class__.__name__} register {type}:{production.__class__.__module__}.{production.__class__.__name__}')
            self.productions[type] = production

    def create(self, type: any) -> FactoryProduction | None:
        """生成产品

        Args:
            type (Any): 产品类型

        Returns:
            FactoryProduction | None: 返回产品
        """
        production: any | None = self.productions.get(type)
        logger.info(f'create type {type} production {production}')
        return production




# -*- coding: utf-8 -*-

import requests
from icemammoth_common.util.log_util import logger
from icemammoth_common.decorator.retry import retry

@retry(tries=3)
def request_data(url='https://thanos-raw.smap.shopee.io/api/v1/query_range', params=None):
    # sourcery skip: raise-specific-error
    if params is None:
        params = {}
    response = requests.get(url, params=params,timeout=(60,120))
    logger.debug('response.status_code = %s',response.status_code)
    if response.status_code != 200:
        try:
            response_json = response.json()
            status = response_json['status']
            error_type = response_json['errorType']
            error = response_json['error']
        except Exception as err:
            logger.exception('query prometheus error!url:%s,params:%s',url,params)
            raise Exception(
                f'unexpected error happend!err:{err},status:{response.status_code},response_text:{response.text}'
            ) from err
        else:
            logger.warning('query prometheus error!url:%s,params:%s,status:%s,error_type:%s,error:%s',url,params,status,error_type,error)
            raise Exception(f'query prometheus error!url:{url},params:{params},status:{status},error_type:{error_type},error:{error}')

    return response.json()


# -*- coding: utf-8 -*-

import time
from datetime import date, timedelta, datetime, timezone


DEFAULT_DATE_TIME_FORMAT = '%Y-%m-%d %H:%M:%S'


def get_start_end_duration(dt="2021-05-05"):
    """[get (time - 2h, time + 27h)]

    Args:
        dt (str, optional): [description]. Defaults to "2021-05-05".

    Returns:
        [tuple(int)]
    """
    dt = date.fromisoformat(dt)
    tmr_dt = dt + timedelta(days=1)
    return int(time.mktime(date.timetuple(dt))),  int(time.mktime(date.timetuple(tmr_dt)) + 3 * 3600)


def get_short_start_end_duration(dt):
    """[get (time, time + 3h)]

    Returns:
        [tuple(int)]
    """
    dt = date.fromisoformat(dt)
    return int(time.mktime(date.timetuple(dt))),  int(time.mktime(date.timetuple(dt)) + 3 * 3600)


def get_now_time():
    return datetime.now().isoformat().split(".")[0]


def set2string(s):
    """[transform set to string]

    Args:
        s ([set])

    Returns:
        [string]
    """
    result = ''
    for i, k in enumerate(s):
        if i > 0:
            result += '\n'
        result += k
    return result


def get_current_unix_timestamp():
    return int(time.time())


def datetime_to_str(date_time: datetime, format=DEFAULT_DATE_TIME_FORMAT) -> str:
    if date_time is None:
        return None
    return date_time.strftime(format)


def str_to_datetime(date_time_str, format=DEFAULT_DATE_TIME_FORMAT) -> datetime:
    if date_time_str is None:
        return None
    return datetime.strptime(date_time_str, format)


def convert_unixtime_to_datetime(unixtimestmap) -> datetime:
    tz = timezone(timedelta(0, 28800))
    return datetime.fromtimestamp(unixtimestmap, tz=tz)


def datetime_add(time, days: int = 0, seconds: int = 0, microseconds: int = 0, milliseconds: int = 0, minutes: int = 0, hours: int = 0, weeks: int = 0) -> datetime:
    return time + timedelta(days, seconds, microseconds, milliseconds, minutes, hours, weeks)


def str_datetime_add(timeStr: str, days: int = 0, seconds: int = 0, microseconds: int = 0, milliseconds: int = 0, minutes: int = 0, hours: int = 0, weeks: int = 0, format: str = DEFAULT_DATE_TIME_FORMAT) -> str:
    time = datetime.strptime(timeStr, format)
    time: datetime = datetime_add(time=time, days=days, seconds=seconds, microseconds=microseconds,
                                  milliseconds=milliseconds, minutes=minutes, hours=hours, weeks=weeks)
    return time.strftime(format)


def get_week(time: datetime) -> int:

    if time is None:
        return None

     # 计算周一是一年中的第几天
    monday: datetime = time - timedelta(days=time.weekday())

    # 计算当前是今年的第几周
    # ISO 8601标准，周一是一周的开始
    return monday.isocalendar()[1]


def get_week_begin_date(time: datetime) -> str:

    if time is None:
        return None

    # 计算周一是一年中的第几天
    monday: datetime = time - timedelta(days=time.weekday())

    # 格式化周起始日期
    return monday.strftime('%Y-%m-%d')



# -*- coding: utf-8 -*-

import random
import time
import traceback
from icemammoth_common.util.log_util import logger as Logger
from functools import partial

from decorator import decorator


def __retry_internal(
    f,
    exceptions=Exception,
    tries=-1,
    delay=0,
    max_delay=10,
    backoff=1,
    jitter=0,
    logger=Logger,
    log_traceback=False,
    on_exception=None,
):
    """
    Executes a function and retries it if it failed.

    :param f: the function to execute.
    :param exceptions: an exception or a tuple of exceptions to catch. default: Exception.
    :param tries: the maximum number of attempts. default: -1 (infinite).
    :param delay: initial delay between attempts. default: 0.
    :param max_delay: the maximum value of delay. default: None (no limit).
    :param backoff: multiplier applied to delay between attempts. default: 1 (no backoff).
    :param jitter: extra seconds added to delay between attempts. default: 0.
                   fixed if a number, random if a range tuple (min, max)
    :param logger: logger.warning(fmt, error, delay) will be called on failed attempts.
                   default: retry.logging_logger. if None, logging is disabled.
    :param on_exception: handler called when exception occurs. will be passed the captured
                         exception as an argument. further retries are stopped when handler
                         returns True. default: None
    :returns: the result of the f function.
    """
    _tries, _delay = tries, delay
    while _tries:
        try:
            return f()
        except exceptions as e:
            if on_exception is not None and on_exception(e):
                raise e

            _tries -= 1
            if _tries <= 0:
                raise e

            try:
                func_qualname = f.func.__qualname__
            except AttributeError:
                func_qualname = str(f.func)
            logger.warning(
                f'method {f.func.__module__}.{func_qualname} execute failed, ' + \
                f'will do {tries-_tries}th retry in {_delay} seconds. ' + \
                f'error_type:{e.__class__.__qualname__}, error_message:{e}'
            )

            if log_traceback:
                logger.warning(traceback.format_exc())

            time.sleep(_delay)
            _delay *= backoff

            _delay += random.uniform(*jitter) if isinstance(jitter, tuple) else jitter
            if max_delay is not None:
                _delay = min(_delay, max_delay)


def retry(
    exceptions=Exception,
    tries=-1,
    delay=0,
    max_delay=10,
    backoff=1,
    jitter=0,
    logger=Logger,
    log_traceback=False,
    on_exception=None,
):
    """Returns a retry decorator.

    :param exceptions: an exception or a tuple of exceptions to catch. default: Exception.
    :param tries: the maximum number of attempts. default: -1 (infinite).
    :param delay: initial delay between attempts. default: 0.
    :param max_delay: the maximum value of delay. default: None (no limit).
    :param backoff: multiplier applied to delay between attempts. default: 1 (no backoff).
    :param jitter: extra seconds added to delay between attempts. default: 0.
                   fixed if a number, random if a range tuple (min, max)
    :param logger: logger.warning(fmt, error, delay) will be called on failed attempts.
                   default: retry.logging_logger. if None, logging is disabled.
    :param on_exception: handler called when exception occurs. will be passed the captured
                         exception as an argument. further retries are stopped when handler
                         returns True. default: None
    :returns: a retry decorator.
    """

    @decorator
    def retry_decorator(f, *fargs, **fkwargs):
        args = fargs or []
        kwargs = fkwargs or {}
        return __retry_internal(
            partial(f, *args, **kwargs),
            exceptions,
            tries,
            delay,
            max_delay,
            backoff,
            jitter,
            logger,
            log_traceback,
            on_exception,
        )

    return retry_decorator


def retry_call(
    f,
    fargs=None,
    fkwargs=None,
    exceptions=Exception,
    tries=-1,
    delay=0,
    max_delay=10,
    backoff=1,
    jitter=0,
    logger=Logger,
    log_traceback=False,
    on_exception=None,
):
    """
    Calls a function and re-executes it if it failed.

    :param f: the function to execute.
    :param fargs: the positional arguments of the function to execute.
    :param fkwargs: the named arguments of the function to execute.
    :param exceptions: an exception or a tuple of exceptions to catch. default: Exception.
    :param tries: the maximum number of attempts. default: -1 (infinite).
    :param delay: initial delay between attempts. default: 0.
    :param max_delay: the maximum value of delay. default: None (no limit).
    :param backoff: multiplier applied to delay between attempts. default: 1 (no backoff).
    :param jitter: extra seconds added to delay between attempts. default: 0.
                   fixed if a number, random if a range tuple (min, max)
    :param logger: logger.warning(fmt, error, delay) will be called on failed attempts.
                   default: retry.logging_logger. if None, logging is disabled.
    :param on_exception: handler called when exception occurs. will be passed the captured
                         exception as an argument. further retries are stopped when handler
                         returns True. default: None
    :returns: the result of the f function.
    """
    args = fargs or []
    kwargs = fkwargs or {}
    return __retry_internal(
        partial(f, *args, **kwargs),
        exceptions,
        tries,
        delay,
        max_delay,
        backoff,
        jitter,
        logger,
        log_traceback,
        on_exception,
    )



# -*- coding: utf-8 -*-

from string import Formatter

class NamespaceFormatter(Formatter):

   def __init__(self) -> None:
       Formatter.__init__(self)

   def get_value(self, key, args, kwargs):# -> Any:
      new_args = []
      for arg in args:
         if arg is None:
            new_args.append('')
         else:
            new_args.append(arg)

      for _key,value in kwargs.items():
         if value is None:
            kwargs[_key] = ''

      return Formatter.get_value(self, key, new_args, kwargs)

formatter = NamespaceFormatter()

def strip(s) -> str:
   return s if s is None or not isinstance(s, str) else s.strip()

def camel_to_underscore(s) -> str:
    """转下划线命名"""
    return ''.join(f'_{c.lower()}' if c.isupper() else c for c in s)

def underscore_to_camel(s) -> str:
    """转驼峰法命名"""
    camelChars = []
    behindUnderScore = False
    for _, c in enumerate(s, 1):
        if c == '_':
            behindUnderScore = True
            continue
        camelChars.append(c.upper()) if behindUnderScore else camelChars.append(c)
        behindUnderScore = False
    return ''.join(camelChars)

def remove_from_first(s:str, tag:str) -> str:
      last_tag_index = s.find(tag)
      if last_tag_index == -1:
          return s
      return s[:last_tag_index]
def remove_from_last(s:str, tag:str) -> str:
      last_tag_index = s.rfind(tag)
      if last_tag_index == -1:
          return s
      return s[:last_tag_index]
def remove_to_first(s:str, tag:str) -> str:
      first_tag_index = s.find(tag)
      if first_tag_index == -1:
          return s
      return s[first_tag_index+1:]
def remove_to_last(s:str, tag:str) -> str:
      first_tag_index = s.rfind(tag)
      if first_tag_index == -1:
          return s
      return s[first_tag_index+1:]


# 以下是添加中文注释后的完整代码
import importlib.util
import sys

def module_exist(module_name:str) -> bool:
    '''判断模块是否存在'''
    return importlib.util.find_spec(module_name) is not None

def get_variable(module_name:str, variable_name:str, default_value:object = None, except_unexist :bool = False) -> object | None:
    '''获取模块中的变量'''
    module = sys.modules.get(module_name)
    if module is None:
        if except_unexist:
            raise ValueError(f"Module '{module_name}' does not exist")
        else:
            return default_value
        
    if hasattr(module, variable_name):
        return getattr(module,variable_name)
    elif except_unexist:
        raise AttributeError(f"Variable '{variable_name}' not found in module '{module_name}'")
    else:
        return default_value

def has_variable(module_name:str, variable_name:str) -> bool:
    '''判断模块中是否存在变量'''
    module = sys.modules.get(module_name)
    return module is not None and hasattr(module, variable_name)


# -*- coding: utf-8 -*-

from typing import Any, Tuple
import requests
from enum import Enum
from requests import Response
from requests.structures import CaseInsensitiveDict
from icemammoth_common.util.log_util import logger
from icemammoth_common.decorator.retry import retry

class ResponseType(Enum):
    TEXT = 'TEXT'
    JSON = 'JSON'
    BYTES = 'BYTES'

DEFAULT_HEADER: dict[str, str] = {
        'accept': '*/*',
        'accept-encoding': 'gzip, deflate, br',
        'accept-language': 'en-US,en;q=0.9,zh-TW;q=0.8,zh-CN;q=0.7,zh;q=0.6',
        'content-type': 'application/json',
        'sec-ch-ua': '"Not?A_Brand";v="8", "Chromium";v="108", "Google Chrome";v="108"',
        'sec-ch-ua-mobile': '?0',
        'sec-ch-ua-platform': '"macOS"',
        'sec-fetch-dest': 'empty',
        'sec-fetch-mode': 'cors',
        'sec-fetch-site': 'same-origin',
        'user-agent': 'Mozilla/5.0 (Macintosh; Intel Mac OS X 10_15_7) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/108.0.0.0 Safari/537.36',
    }

def post(url, headers:dict[str,str]=DEFAULT_HEADER, data = None, responseType:ResponseType = ResponseType.TEXT) -> Tuple[int, CaseInsensitiveDict[str], Any]:
    return send_http_request(url, 'post', headers = headers, data = data, responseType = responseType)

def get(url, headers:dict[str,str]=DEFAULT_HEADER, params:dict[str,str] = None, responseType:ResponseType = ResponseType.TEXT) -> Tuple[int, CaseInsensitiveDict[str], Any]:
    return send_http_request(url, 'get' , headers = headers, params = params, responseType = responseType)

def handle_exception(exception):
    if type(exception) == FileNotFoundError:
        return True
    return False

@retry(tries=3,on_exception=handle_exception)
def send_http_request(url, method:str='GET', **kwargs) -> Tuple[int, CaseInsensitiveDict[str], Any]:
    '''
        responseType can be:
            TEXT(default): return response context as a string 
            JSON: return response context as a json
            BYTES: return response context as byte array
    '''
    legalMethods: tuple = ('GET', 'POST')
    if not method or method.upper() not in legalMethods:
        raise TypeError(f'requestType type {method} is not one of legal requestType types {legalMethods}')
    responseType = kwargs.pop('responseType')
    if not responseType or type(responseType) is not ResponseType:
        raise TypeError(f'param responseType must not be none and type must be ResponseType!responseType:{responseType}')
    try:
        logger.debug(f'request url={url},method=,{method},{kwargs}')
        kwargs.setdefault('timeout',(60, 120))
        response: Response = requests.request(method.lower(), url, **kwargs)

        statusCode: int = response.status_code
        header: CaseInsensitiveDict[str] = response.headers

        # 409 Conflict: conflict with the current state of the target resource. 
        if int(statusCode/100) == 5 or statusCode in [403,409]:
            logger.exception(f'http request filed!url:{url},method:{method},{kwargs},responseCode:{response}, failedReason:{response.reason}')
            raise Exception(response)
                
        # Unauthorized
        if statusCode == 401:
            logger.exception(f'http request unauthorized!url:{url},method:{method},{kwargs},responseCode:{response}, failedReason:{response.reason}')
            raise Exception(response)

        # Resource Not Found
        if statusCode == 404:
            logger.exception(f'http request not found file!url:{url},method:{method},{kwargs},responseCode:{response}, failedReason:{response.reason}')
            raise FileNotFoundError(response)
            
        if responseType == ResponseType.TEXT:
            return statusCode,header,response.text
        elif responseType == ResponseType.JSON:
            return statusCode,header,response.json()
        elif responseType == ResponseType.BYTES:
            return statusCode,header,response.content
        else:
            raise TypeError(f'param responseType must not be none and type must be ResponseType!responseType:{responseType}')
    except Exception as err:
        logger.exception(f'send http request error!url:{url},method:{method},{kwargs}')
        raise err



from collections import Counter

def mostMember(collection):
    counter = Counter(collection)
    return counter.most_common(1)[0][0]

def find_one(collection, func):
    for item in collection:
        if func(item):
            return item
    return None

def reorder_list_by_standard_order(originList, standardOrderList):
    return [item for item in standardOrderList for originItem in originList if item == originItem]



# -*- coding: utf-8 -*-

import threading
from decorator import decorator
from icemammoth_common.util.log_util import logger
from icemammoth_common.database.connect_pool import ConnectPool

'''
    ThreadLocal.local每次调用都会返回一个新对象,并非线程内唯一
    作为全局变量方便实现线程的内变量共享,放到函数体内则在函数每次执行时会生成不同的变量,无法实现线程的内变量共享
    这点与Java中的ThreadLocal相同
'''
local = threading.local()

class TransactionManager(object):
    
    connection = None
    inTransaction = False
    
    def __init__(self):
        self.connectionPool = ConnectPool()
    
    def startTx(self):
        TransactionManager.connection = self.connectionPool.getConnection()
        TransactionManager.connection.start_transaction()
        TransactionManager.inTransaction = True
    
    def commit(self):
        try:
            TransactionManager.connection.commit()
            TransactionManager.connection.close()
        finally:
            TransactionManager.connection = None
            TransactionManager.inTransaction = False
    
    def rollback(self):
        try:
            TransactionManager.connection.rollback()
            TransactionManager.connection.close()
        finally:
            TransactionManager.connection = None
            TransactionManager.inTransaction = False
            
def transaction():
    @decorator
    def transaction_decorator(f, *fargs, **fkwargs):
        firstLevel = False
        # transactionManager = None
        try:
            local.transactionManager
        except AttributeError as error:
            logger.debug('no transactionManager in thread local')
            logger.debug('enter transaction scope')
            local.transactionManager = None

        if not local.transactionManager:
            firstLevel = True
            local.transactionManager = TransactionManager()
            # transactionManager = local.transactionManager
        
        try:
            if firstLevel:
                logger.debug(msg='start transaction')
                local.transactionManager.startTx()
            result = f(*fargs,**fkwargs)
            if firstLevel:
                logger.debug('commit transaction')
                local.transactionManager.commit()
            return result
        except Exception as exception:
            if firstLevel and local.transactionManager.inTransaction:
                logger.exception(f'transaction execute failed, rollback!func:{f.__module__}:{f.__qualname__}')
                local.transactionManager.rollback()
            raise exception
        finally:
            if firstLevel:
                logger.debug('exit transaction scope')
                local.transactionManager = None
        
    return transaction_decorator


import os
from setuptools import setup, find_packages



# 读取依赖文件
current_dir = os.path.dirname(os.path.abspath(__file__))
requirements_path = os.path.join(current_dir, "requirements.txt")
readme_path = os.path.join(current_dir, "README.md")
with open(requirements_path, "r", encoding="utf-8") as f:
    install_requires = f.read().splitlines()

setup(
    name='icemammoth_common',
    version='0.1.0.dev.1',
    # find_packages()会自动发现项目中的所有包和子包。
    # 默认情况下，find_packages() 会递归地查找所有包含 __init__.py 的目录，并将它们视为包。where表示从哪个目录开始搜索
    packages=find_packages(where=".",include=["icemammoth_common","icemammoth_common.*"], exclude=["test", "test.*"]),
    # 指定包的源代码位置, 控制包的安装路径: package_dir = {"<目标路径>": "<源代码路径>"}
    # 目标路径：表示包在安装后应该被放置的目录结构。通常是一个空字符串 ""，表示项目的根目录。
    # 源代码路径：表示包的源代码实际存放的位置。
    # 一般不需要设置，因为默认情况下，Python 会自动将包名作为模块名，并使用点号（.）分隔。
    # package_dir={"icemammoth_common": "icemammoth_common"},          
    description='common tools collection',
    long_description=open(readme_path).read(),
    long_description_content_type='text/markdown',
    author='Klein',
    author_email='myicemammoth@gmail.com',
    url='',
    # 许可证类型
    license='MIT', 
    classifiers=[
        # 包分类列表，例如：
        'Development Status :: 3 - Alpha',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: MIT License',
        'Programming Language :: Python :: 3.12',
    ],
    keywords='common utils',
    install_requires= install_requires, # 依赖项（从 requirements.txt 读取）
    # 如果你的模块包含数据文件，可以在这里添加 package_data 字段
    # package_data={
    #     'common': ['data/*.dat'],
    # },
    # 如果你的模块包含二进制文件，可以在这里添加 ext_modules 字段
    # ext_modules=[
    #     # 你的扩展模块
    # ],
    # 测试配置（可选）
    # test_suite="test",  # 指定测试模块
    # tests_require=["pytest"],  # 测试依赖
)


# -*- coding: utf-8 -*-

import inspect
import xml.etree.ElementTree as ET
from util import object_util


class DefaultTagGenerator:
    
    def generateTag(self,tag):
        return tag[0].upper() + tag[1:]
        
    def generateListMemberTag(self,listTag):
        return 'Item'
    
def convertObjToXMLElement(obj, eleTag, parentTag=None, tagGenerator=DefaultTagGenerator()):
    
    if eleTag is None:
        raise ValueError("eleTag can not be None!")

    generatedEleTag = tagGenerator.generateTag(eleTag)
    if parentTag is not None:
        generatedGarentTag = tagGenerator.generateTag(parentTag)
        ele = ET.Element(generatedGarentTag)
        subEle = convertObjToXMLElement(obj,generatedEleTag,tagGenerator=tagGenerator)
        ele.append(subEle)
        return ele
    
    ele = ET.Element(generatedEleTag)
    if isinstance(obj,str):
        ele.text = obj
    elif isinstance(obj,list) or isinstance(obj,tuple) or isinstance(obj, set) or isinstance(obj, frozenset):
        listMemberTag = tagGenerator.generateListMemberTag(eleTag)
        for v in obj:
            subEle = convertObjToXMLElement(v, listMemberTag,tagGenerator=tagGenerator)
            ele.append(subEle)
    elif isinstance(obj,int) or isinstance(obj,float) or isinstance(obj,complex) or \
        isinstance(obj, bool) or isinstance(obj, bytes) or isinstance(obj, bytearray):
        ele.text = str(obj)
    elif isinstance(obj, dict):
        for k,v in obj.items():
            subEle = convertObjToXMLElement(v,k,tagGenerator=tagGenerator)
            ele.append(subEle)
    elif inspect.isclass(obj.__class__):
        for k,v in object_util.getObjectAttributesAndValues(obj).items():
            subEle = convertObjToXMLElement(v,k,tagGenerator=tagGenerator)
            ele.append(subEle)
    else:
        raise ValueError(f'unkown data type {type(obj)}')

    return ele

def convertObjToXMLString(obj, eleTag, parentTag=None, tagGenerator=DefaultTagGenerator()):

    ele = convertObjToXMLElement(obj,eleTag,parentTag,tagGenerator)
    ET.indent(ele, space='  ', level=0)
    return str(ET.tostring(ele,encoding='unicode',method='xml'))


# -*- coding: utf-8 -*-

from icemammoth_common.util.log_util import logger

from decorator import decorator

class NotExistError(Exception): ...
class ParamError(Exception): ...
class InvalidTypeError(Exception): ...
class OperationError(Exception): ...
class OverLimitError(Exception): ...
class ExecuteError(Exception): ...
class ConcurrentError(Exception): ...

def catch_exception( msg:str = None):

    @decorator
    def catch_exception_decorator(f, *fargs, **fkwargs):
        args = fargs or []
        kwargs = fkwargs or {}
        return __process_exception(
            f,
            msg,
            *args,
            **kwargs
        )
        

    return catch_exception_decorator

def __process_exception(f,msg:str = None , *args, **kwargs):# -> Any:
    try:
        return f(*args,**kwargs)
    except Exception as e:
        if not msg:
            msg = f'execute func {f.__qualname__} in module {f.__module__} failed!'
        logger.exception(f'{msg}')
        raise e


# -*- coding: utf-8 -*-

from os import path

"""
    path中生成最终路径的函数有realpath(),abspath()和normpath()
    realpath(): 返回canonical path, 对于链接还原为真实目录,函数实现依赖abspath()
    abspath():  返回absolute path,  函数实现依赖,normpath()
    normpath(): 返回normalization path, 表示常用的目录表达方式,去除双斜杠,相对路径等,其路径中可能包含链接
    
    Canonical path: 规范路径,将路径中的link转化为所连接的真实路径
                    The canonical path is always an absolute and unique path
    Absolute path:  绝对路径,路径中包含从根目录到达该目录或文件的所有目录和文件信息,不进行link转换
                    An absolute path is defined as the specifying the location 
                    of a file or directory from the root directory(/). 
                    In other words we can say absolute path is a complete path 
                    from start of actual filesystem from / directory.
    Normalization path: 标准路径,去除双斜杠,单点,双点等相对路径符号,不要求从根目录开始,可以包含link
                    Normalize a path, e.g. A//B, A/./B and A/foo/../B all become A/B.
                    It should be understood that this may change the meaning of the path
                    if it contains symbolic links!
"""

def create_real_path(*paths):
    return path.realpath(path.join(*paths))

def create_abs_path(*paths):
    return path.abspath(path.join(*paths))

def create_norm_path(*paths):
    return path.norm(path.join(*paths))


# -*- coding: utf-8 -*-

from bs4 import BeautifulSoup

def fetchImageSrcs(html) -> list[str]:
    soup = BeautifulSoup(html, "lxml")
    # Find all images element using find_all()
    images = soup.find_all("img")
    return [
        image.get('src')
        for image in images
    ]

def replaceImageSrc(html,oriSrc,destSrc) -> str:
    soup = BeautifulSoup(html, "lxml")
    # Find all images element using find_all()
    images = soup.find_all("img")
    for image in images:
        if image.get('src') == oriSrc:
            image['src'] = destSrc
    return soup.prettify()

def removeImageBySrc(html,src) -> str:
    soup = BeautifulSoup(html, "lxml")
    # Find all images element using find_all()
    images = soup.find_all("img")
    for image in images:
        if image.get('src') == src:
            image.replace_with()
    return soup.prettify()




# -*- coding: utf-8 -*-

class BaseObject(object):
    
    def __repr__(self) -> str:
        return self.__str__()
    
    def __str__(self) -> str:
        attributes = [
            attribute
            for attribute in dir(self)
            if not attribute.startswith("__") and not callable(getattr(self, attribute))
        ]
        attributeMap = {
            attribute: str(getattr(self, attribute)) for attribute in attributes
        }
        return f"{attributeMap}" 


# -*- coding: utf-8 -*-

from typing import Any, Dict, List, Tuple, TypeVar, Generic
from icemammoth_common.base import BaseObject

from icemammoth_common.database.connect_pool import ConnectPool
from icemammoth_common.error.error import NotExistError
from .transaction import TransactionManager
from icemammoth_common.util.log_util import logger
from icemammoth_common.util import string_util, object_util

class ModelAttrValueConvertor(object):

    def attrValueToColumnValue(self, table, attrName, attrValue):
        pass

    def columnValueToAttrValue(self, table, columnName, columnValue):
        pass


class DAOInheritException(BaseException):

    def __init__(self, msg):
        super(msg)


class ORMConfig(BaseObject):

    def __init__(
        self,
        table: str = None,
        # columns: [str] = None,
        model=None,
        columnAttrMap: Dict[str, str] = None,
        attrValueConvertor: ModelAttrValueConvertor = None,
    ):
        self.table = table
        self.model = model
        self.columnAttrMap = columnAttrMap
        self.attrValueConvertor = attrValueConvertor


# 使用泛型,必须先使用TypeVar定义泛型类型
ModelType = TypeVar("ModelType", bound=BaseObject)

def defaultProcessFunc(cursor) :
    return cursor.fetchall()


class BaseDAO(Generic[ModelType]):

    def __init__(self):
        if not self.__ORM_CONFIG__:
            raise DAOInheritException(
                f"class {
                    self.__class__.__name__} inherit from BaseDAO must set attribute __ORM_CONFIG__"
            )
        self.ormConfig = self.__ORM_CONFIG__
        self.table = self.ormConfig.table
        self.modelType = self.ormConfig.model
        self.modelAttrs = object_util.fetchObjectAttributes(self.ormConfig.model())
        self.attrColumnMap = {}
        self.columnAttrMap = {}
        self.mappedColumns = []
        self.mappedAttrs:List[str] = []
        for column, attr in self.ormConfig.columnAttrMap.items():
            if not attr:
                attr = string_util.underscore_to_camel(column)
            if attr not in self.modelAttrs:
                raise NotExistError(
                    f"dao {self.__class__.__name__} orm config error, class {self.ormConfig.model.__name__} doesn't exist attribute {
                        attr} which is mapped with column {self.table}.{column}"
                )
            self.attrColumnMap[attr] = column
            self.columnAttrMap[column] = attr
            self.mappedColumns.append(column)
            self.mappedAttrs.append(attr)

        self.attrValueConvertor = self.ormConfig.attrValueConvertor
        self.connectionPool = ConnectPool()
        self.transactionManager = TransactionManager()

    def _fetchValuesByAttrs(self, model, attrs=None) -> List[Any]:
        row = []

        attrValueMap = object_util.getObjectAttributesAndValues(model)
        attrs = attrs if attrs else self.mappedAttrs

        for attr in attrs:
            value = attrValueMap.get(attr)
            value = self.attrValueConvertor.attrValueToColumnValue(
                self.table, attr, value) if self.attrValueConvertor else value
            row.append(value)

        return row
    def fillRowDataToModel(self, columns: List[str], row: List[Any], model):

        if row is None or columns is None:
            return None

        for idx, column in enumerate(columns):
            attr = self._fetchAttrByColum(column)
            if not attr:
                raise NotExistError(
                    f"dao {self.__class__.__name__} convert row to model failed, class {
                        self.ormConfig.model.__name__} no attribute mapped with column {self.table}.{column}."
                )
            value = row[idx]
            value = self.attrValueConvertor.columnValueToAttrValue(
                self.table, column, value) if self.attrValueConvertor else value
            setattr(model, attr, value)
        return model

    def _insertData(self, cursor):
        return cursor.lastrowid

    def _insertDatas(self, cursor):
        return (cursor.lastrowid, cursor.rowcount)

    def _readDatas(self, cursor):
        return (
            [
                self.fillRowDataToModel(
                    cursor.column_names, result, self.ormConfig.model())
                for result in results
            ]
            if (results := cursor.fetchall())
            else None
        )

    def _readData(self, cursor):
        result = cursor.fetchone()
        return self.fillRowDataToModel(cursor.column_names, result, self.ormConfig.model())

    def _deleteDatas(self, cursor):
        return cursor.rowcount

    def _updateDatas(self, cursor):
        return cursor.rowcount

    def execute(self, sql: str, params:List[Any], process_func=None, executeMany=False):
        try:
            connection = None
            if self.transactionManager.inTransaction:
                connection = self.transactionManager.connection
            else:
                connection = self.connectionPool.getConnection()
            cursor = connection.cursor()
            if executeMany:
                cursor.executemany(sql, params)
            else:
                cursor.execute(sql, params)
        
            result = None
            if process_func is None:
                result = defaultProcessFunc(cursor)
            else:
                result = process_func(cursor)
            cursor.close
            return result
        except Exception as e:
            logger.exception(
                f"execute sql failed!sql:{sql},params:{params},executeMany:{executeMany}"
            )
            raise e
        finally:
            if not self.transactionManager.inTransaction and connection:
                connection.close()

    def insertData(self, model: ModelType, ignoreDuplicate=False, duplicateUpdateAttrs: List[str] = [], duplicateUpdateTerms: List[str] = []) -> int:
        sql: str = self._generateInsertSql(ignoreDuplicate=ignoreDuplicate, duplicateUpdateAttrs=duplicateUpdateAttrs, duplicateUpdateTerms=duplicateUpdateTerms)
        logger.debug(f"sql is {sql}")
        params: List[Any] = self._fetchValuesByAttrs(model)
        return self.execute(sql, params, self._insertData)

    def insertDatas(self, models: List[ModelType], ignoreDuplicate=False, duplicateUpdateAttrs: List[str] = [], duplicateUpdateTerms: List[str] = []) -> Tuple[int]:
        sql: str = self._generateInsertSql(ignoreDuplicate=ignoreDuplicate, duplicateUpdateAttrs=duplicateUpdateAttrs, duplicateUpdateTerms=duplicateUpdateTerms)
        logger.debug(f"sql is {sql}")
        params: List[List[Any]] = [self._fetchValuesByAttrs(model) for model in models]
        return self.execute(sql, params, self._insertDatas, True)

    def insertDatasByBatch(self, models: List[ModelType], ignoreDuplicate=False, duplicateUpdateAttrs: List[str] = [], duplicateUpdateTerms: List[str] = [], pageSize:int=1000) -> Tuple[int,int]:
        sql: str = self._generateInsertSql(ignoreDuplicate=ignoreDuplicate, duplicateUpdateAttrs=duplicateUpdateAttrs, duplicateUpdateTerms=duplicateUpdateTerms)
        logger.debug(f"sql is {sql}")
        result: Tuple[int,int] = [0, 0]
        start: int = 0
        while start < len(models):
            end: int = min(start + pageSize, len(models))
            _models: List[Any] = models[start:end]
            params: List[List[Any]] = [self._fetchValuesByAttrs(model) for model in _models]
            _result = self.execute(sql, params, self._insertDatas, True)
            result = [x + y for x, y in zip(result, _result)]
            start = end
        
        return result
    def _generateInsertSql(self, insertAttrs: List[str] = None, ignoreDuplicate=False, duplicateUpdateAttrs: List[str] = [], duplicateUpdateTerms: List[str] = []):
        columns = []
        if insertAttrs:
            for attr in insertAttrs:
                column = self._fetchColumnByAttr(attr)
                columns.append(column)
        else:
            columns = self.mappedColumns
        columnsSQL = ",".join(columns)
        ignoreDuplicateTag = 'IGNORE' if ignoreDuplicate else ''
        valuesSQL = ','.join(['%s']*len(columns))

        duplicateUpdateSQL: str = ""
        if not ignoreDuplicate:
            
            if duplicateUpdateAttrs:

                duplicateUpdateSQL = 'ON DUPLICATE KEY UPDATE'
                for attr in duplicateUpdateAttrs:
                    column = self._fetchColumnByAttr(attr)
                    duplicateUpdateSQL = f'{duplicateUpdateSQL} {column} = VALUES({column}), '

            if duplicateUpdateTerms:
                for term in duplicateUpdateTerms:
                    duplicateUpdateSQL = f'{duplicateUpdateSQL} {term}, '

            if duplicateUpdateSQL[-2:] == ', ':
                duplicateUpdateSQL = duplicateUpdateSQL[:-2]

        return f"INSERT {ignoreDuplicateTag} INTO {self.table} ({columnsSQL}) VALUES ({valuesSQL}) {duplicateUpdateSQL}"

    def readData(self, readAttrs: List[str] = None, **conditions) -> ModelType:
        sql = self._generateReadSql(
            readOne=True, readAttrs=readAttrs, **conditions)
        params = list(conditions.values())
        return self.execute(sql, params, self._readData)

    def readDatas(self, readAttrs: List[str] = None, **conditions) -> List[ModelType]:
        sql = self._generateReadSql(
            readOne=False, readAttrs=readAttrs, **conditions)
        params = list(conditions.values())
        return self.execute(sql, params, self._readDatas)

    def readDatasByConditionSQL(
        self, readAttrs: List[str] = None, conditionSql: str = None, params = None, orderBy:str = None, limit:int = None, offset:int = None
    ) -> List[ModelType]:

        # 定义一个空列表，用于存储要读取的列
        readColumns = []
        # 如果readAttrs不为空，则遍历readAttrs，将每个属性对应的列添加到readColumns中
        if readAttrs:
            for attr in readAttrs:
                column = self._fetchColumnByAttr(attr)
                readColumns.append(column)
        # 如果readAttrs为空，则将mappedColumns赋值给readColumns
        else:
            readColumns = self.mappedColumns
        columnsSQL: str = ",".join(readColumns)

        selectSQL = f"SELECT {columnsSQL} FROM {self.table}"
        
        if conditionSql:
            selectSQL = f"{selectSQL} WHERE {conditionSql}"
        
        if orderBy:
            selectSQL = f"{selectSQL} ORDER BY {orderBy}"

        if limit:
            selectSQL = f"{selectSQL} LIMIT {limit}"

        if offset:
            selectSQL = f"{selectSQL} OFFSET {offset}"
        
        return self.execute(selectSQL, params, self._readDatas)

    def _generateReadSql(self, readOne: bool = False, readAttrs: List[str] = None, **conditions):

        params = []

        readColumns = []
        if readAttrs:
            for attr in readAttrs:
                column = self._fetchColumnByAttr(attr)
                readColumns.append(column)
        else:
            readColumns = self.mappedColumns
        columnsSQL: str = ",".join(readColumns)

        conditionColumns = []
        for attr, value in conditions.items():
            column = self._fetchColumnByAttr(attr)
            conditionColumns.append(column)
            value = self.attrValueConvertor.attrValueToColumnValue(
                self.table, attr, value) if self.attrValueConvertor else value
            params.append(value)
        conditionSQL: str = " and ".join(
            [f"{column} = %s" for column in conditionColumns])

        limitSQL: str = "LIMIT 1" if readOne else ""

        return f"SELECT {columnsSQL} FROM {self.table} WHERE {conditionSQL} {limitSQL}"

    def deleteDatas(self, **conditions):

        params = []

        conditionColumns = []
        for attr, value in conditions.items():
            column = self._fetchColumnByAttr(attr)
            conditionColumns.append(column)
            value = self.attrValueConvertor.attrValueToColumnValue(
                self.table, attr, value) if self.attrValueConvertor else value
            params.append(value)
        conditionSQL = " AND ".join(
            [f"{column} = %s" for column in conditionColumns])

        deleteSQL = f"DELETE FROM {self.table} WHERE {conditionSQL}"

        return self.execute(deleteSQL, params, self._deleteDatas)

    def updateDatas(self, updateAttrs: Dict[str, str], **conditions):

        updateColumns = []
        params = []

        for attr, value in updateAttrs.items():
            column = self._fetchColumnByAttr(attr)
            updateColumns.append(column)
            columnValue = self.attrValueConvertor.attrValueToColumnValue(
                self.table, attr, value) if self.attrValueConvertor else value
            params.append(columnValue)
        updateColumnSQL = ', '.join(
            [f'{column} = %s' for column in updateColumns])

        conditionColumns = []
        for attr, value in conditions.items():
            column = self._fetchColumnByAttr(attr)
            conditionColumns.append(column)
            columnValue = self.attrValueConvertor.attrValueToColumnValue(
                self.table, attr, value) if self.attrValueConvertor else value
            params.append(columnValue)
        conditionSQL = " AND ".join(
            [f"{column} = %s" for column in conditionColumns])

        updateSQL = f"UPDATE {self.table} SET {
            updateColumnSQL} WHERE {conditionSQL}"

        return self.execute(updateSQL, params, self._updateDatas)

    def _fetchColumnByAttr(self, attr: str):
        column = self.attrColumnMap.get(attr)
        if not column:
            raise TypeError(f'table {self.table} not column bind with attr {
                            self.modelType.__qualname__}.{attr}')
        return column

    def _fetchAttrByColum(self, column):
        attr = self.columnAttrMap.get(column)
        if not attr:
            raise TypeError(f'{self.modelType.__qualname__} not attribute bind with table {
                            self.table} column {column}')
        return attr